/*
Date: 2023/11/14
Auteurs : Louis-Philippe Fortier (FORL79090407), Nathan Pinard (PINN85050400) et Yoan Tremblay (TREY87020409) || groupe 00
Description : Un programme qui fait le calcul de division en utilisant la restauration binaire.
*/

//Imports
#include <iostream>
#include <stdio.h>
#include <limits.h>

//Entête de fonction
unsigned int divisionBinaire(unsigned int a, unsigned int b);
void entreVal(unsigned int* val, unsigned int upperLim, unsigned int lowerLim);
void entreDiv(unsigned int* val1, unsigned int* val2);

int main()
{
    //Déclaration des variables
    unsigned int a, b, ans, nbrDiv;

    FILE* fp;
    errno_t err;

    //Vérifie si le fichier s'ouvre correctement sinon affiche à la console que le fichier ne s'est pas ouvert
    if ((err = fopen_s(&fp, "reponse.txt", "w")) == 0) {
        //Demande du nombre de division que l'utilisateur veut effectuer
        printf("Entrer le nombre de division a effectuer : \n");
        entreVal(&nbrDiv, UINT_MAX, 0);

        //Boucle qui effectue autant de division que l'utilisateur a demandé et les écrit dans un fichier
        for (int i = 1; i <= nbrDiv; i++) {
            printf("Entrer la division %d de la facon suivante : a/b \n", i);
            entreDiv(&a, &b);
            ans = divisionBinaire(a, b);
            fprintf_s(fp, "Division %d: %hu/%hu = %hu reste %d\n", i, a, b, ans, ans >> 16);
        }
        //Vérifie s'il y a des erreurs durant la fermeture du fichier
        err = fclose(fp);
        if (err == 0) {
            printf("Les reponses ont ete ecrites dans le fichier reponse.txt.");
        }
    }
    else {
        printf("Une erreur c'est produite");
    }
    return 0;
}

/*
Entrées: un pointeur vers un entier non signé qui représente la valeur entrée par l'utilisateur
        un entier non signé qui représente la limite suppérieur que la valeur peut prendre
        un entier non signé qui représente la limite inférieur que la valeur peut prendre
Sortie : Rien
Description : Une fonction qui demande a l'utilisateur une valeur tant que cette valeur est invalide
*/
void entreVal(unsigned int* val, unsigned int upperLim, unsigned int lowerLim) {
    //Déclaration des variables
    char c[20];
    long int numLong = 0;
    char* endptr;
    bool isValid = false;
    int ch;

    //Boucle qui demande à l'utilisateur d'entré une valeur valide tant qu'elle n'est pas valide
    do {
        errno = 0;
        printf("Entrer le nombre : ");
        scanf_s("%19s", c, (unsigned int)sizeof(c));
        numLong = strtol(c, &endptr, 10);
        if (*endptr != '\0' || errno == ERANGE || numLong > upperLim || numLong < lowerLim) {
            printf("Erreur : Le nombre doit etre entre %d et %d\n", lowerLim, upperLim);
            while ((ch = getchar()) != '\n' && ch != EOF);
            continue;
        }
        isValid = true;
    } while (!isValid);
    *val = (unsigned int)numLong;
}

/*
Entrée: un pointeur vers un entier non signé qui représente la première valeur entrée par l'utilisateur
        un pointeur vers un entier non signé qui représente la deuxième valeur entrée par l'utilisateur
Sortie: Rien
Description: Une fonction qui demande à l'utilisateur 
*/
void entreDiv(unsigned int* val1, unsigned int* val2) {
    bool isValid = false;
    unsigned int a, b, ch;

    do {
        printf("Entrer la division :");
        scanf_s("%d/%d", &a, &b);
        if (a < 0 || b < 1 || a > USHRT_MAX || b > USHRT_MAX) {
            printf("Entrer invalide, entrez la division sous la forme a/b avec a et b (>1) positif.\n");
            while ((ch = getchar()) != '\n' && ch != EOF);
            continue;
        }
        isValid = true;
    } while (!isValid);
    *val1 = a;
    *val2 = b;
}
/*
Entrée : Un entier non signé qui représente le numérateur de la division
        Un entier non signé qui représente le dénominateur de la division
Sortie: Un entier non signé qui représente le résultat de la division
Description : Une fonction qui calcule le résultat d'une division avec la restauration binaire
*/
unsigned int divisionBinaire(unsigned int a, unsigned int b) {
    b <<= 16;
    for (int i = 0; i < 16; i++) {
        a <<= 1;
        if (a >= b) {
            a = a - b;
            a++;
        }
    }
    return a;
}
